<?php

/**
 * @file
 * Provides a MySQL ORDER BY FIELD() sort option
 */

/**
 * @package Views
 * @subpackage Views MySQL order by field
 * @author Liran Tal <liran.tal@gmail.com>
 */


/**
 * views_mysql_sort_by_field views sort handler.
 * Extends the views_handler_sort class to provide a MySQL ORDER BY FIELD() option to sort by
 */
class views_mysql_sort_by_field extends views_handler_sort {

	/**
	 * Query build.
	 * Adjust the orderby query and bind the order_by_field and order_by_field_text form elements from the settings form to the query.
	 */
	public function query() {

		$field_order = 'FIELD('.$this->options['order_by_field'].', '.$this->options['order_by_field_text'].')';	
		$this->query->add_orderby(NULL, NULL, NULL, $field_order);

	}


	/**
	 * Sort field options form.
	 * Extend the form settings of this sort handler to provide a configurable field name from the fields chosen and a field value.
	 */
	public function options_form(&$form, $form_state) {
		
		parent::options_form($form, $form_state);

		// Get the view name.
		$view = views_get_view($this->view->name);

		// All fields, in all displays - use this instead of the specific page view if you think this is useful 
		/*
		$fields = array();
		foreach($view->display as $display_name => $display_object) {
			$view = views_get_view($this->view->name);
			$view->build($display_name);
			if ($view) {
				$fields = $view->query->fields;
				foreach($fields as $field_name => $field_data) {
					if (empty($field_data['table'])) {
						$fields_name[$field_data['alias']] = $display_name. ' : '.$field_data['alias'];
					} else {
						$fields_name[$field_data['table'].'.'.$field_data['field']] = $display_name. ' : '.$field_data['table'].'.'.$field_data['field'];
					}
				}
			}
		}
		*/

		$view = views_get_view($this->view->name);
		$display_name = $this->view->current_display;
		$view->build($display_name);
		if ($view) {
			$fields = $view->query->fields;
			foreach($fields as $field_name => $field_data) {
				if (empty($field_data['table'])) {
					$fields_name[$field_data['alias']] = $display_name. ' : '.$field_data['alias'];
				} else {
					$fields_name[$field_data['table'].'.'.$field_data['field']] = $display_name. ' : '.$field_data['table'].'.'.$field_data['field'];
				}
			}
		}


		// There's no need for ASC or DESC order in this field type.
		unset($form['order']);

		// Set the default options for form settings
		$order_by_field = isset($form_state['view']->display['default']->display_options['sorts']['views_mysql_sort_by_field']['order_by_field']) ? $form_state['view']->display['default']->display_options['sorts']['views_mysql_sort_by_field']['order_by_field'] : current($fields_name);
		$order_by_field_text = isset($form_state['view']->display['default']->display_options['sorts']['views_mysql_sort_by_field']['order_by_field_text']) ? $form_state['view']->display['default']->display_options['sorts']['views_mysql_sort_by_field']['order_by_field_text'] : '';

		// Extend the form element with the field order_by_field and it's value
    $form['order_by_field'] = array(
      '#title' => t('Order By Field'),
      '#type' => 'select',
      '#options' => $fields_name,
      '#default_value' => $order_by_field,
      '#required' => TRUE,
    );

    $form['order_by_field_text'] = array(
    	'#title' => t('Comma-separated list of values for this field to order by'),
    	'#type' => 'textfield',
    	'#default_value' => $order_by_field_text,
    	'#description' => t('Most probably the field value would be a string so you need to wrap it in single quotes. Example for valid input: \'one\', \'two\', \'three\' '),
    	'#required' => TRUE,
		);

	}


}